#! /usr/bin/env python
"""
--------------------------------------------------------------------------------
Copyright (C) 2005 David Stanczak <zerodivide@users.sourceforge.net>

Released under the GPL, version 2. Except the artwork, which is released
pursuant to the restrictions imposed by their authors.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to
deal in the Software without restriction, including without limitation the
rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
sell copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies of the Software and its documentation and acknowledgment shall be
given in the documentation and software packages that this Software was
used.

THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.   

--------------------------------------------------------------------------------
This was built on python 2.3.4, on a linux 2.6.11-rc2 glibc 2.3.4
x86 machine. Just run this script from its final location on you disk to
start the desklet. See README for details.

*** DO NOT edit this file for changing the configuration; edit instead
`config.txt' in this directory ***

--------------------------------------------------------------------------------
"""
import adesklets
try:
	import statgrab
except ImportError:
	print "ERROR: **** You are missing the pystatgrab module"
	print "       **** Read the README file for dependency information"
	adesklets.quit()

from os import getenv, system
from os.path import join, dirname
from time import sleep

class GeneralConfig(adesklets.ConfigFile):
	"""
---------This is the anetmon config file--------

   border_left,border_right,etc:
      (True or False) turns the border on or off

   device_name:
      This is the name of the network interface you would like to monitor.
      You can always change this value from the menu.
      examples: 'eth0'
                'lo'
		'vmnet8'

   max_KB_in, max_KB_out:
      sets the value in kilobytes at which the gauge will display 100%

   theme:
      points to one of the .theme files
   
	"""
	cfg_default = {	'border_left' : True,
			'border_right' : True,
			'border_top' : True,
			'border_bottom' : True,
			'device_name' : 'eth0',
			'max_KB_in' : 150,
			'max_KB_out' : 150,
			'theme': 'themes/default.theme'}
	
	def __init__(self, id, filename):
		adesklets.ConfigFile.__init__(self, id, filename)

class ThemeConfig(adesklets.ConfigFile):
	"""
   border:  
      points to a directory holding the border images

   effect:
      applies a tint, use None to disable tinting
      examples: 'tint(alpha=128,red=200,green=200,blue=200);'
                None
		'tint(alpha=255,red=255,green=0,blue=0);'   
   
   background_blur: 
      specifies the blur radius
      examples:  '0'  no blur
              : '20'  lots of blur
	
   in_gauge, out_gauge: 
      points to a directory holding the gauge images

   icon:
      points to a icon file 

   font:
      (type/size) Font names are case sensitive.
      For a complete list, type this at a bash prompt 
      echo "list_fonts" | adesklets : 
      Don't forget the trailing colon.

   font_color:
      sets the font color using html style color codes.
      examples: 'FFFFFF'   white 
              : '00FF0080' green with 50% transparency
              : '0000FFFF' blue with 0% transparency 

   separator_color:
      sets the separator color using html style color codes.
	"""
	cfg_default = { 'THEME_AUTHOR': 'UNKNOWN',
			'AUTHOR_CONTACT': 'UNKNOWN',
			'background_blur': '3',
			'background_effect': 'tint(alpha=128,red=200,green=200,blue=200);',
			'border': 'images/borders/sunken2/',
			'border_effect': 'tint(alpha=128,red=200,green=200,blue=200);',
			'font': 'VeraBd/8',
			'font_color': '000000',
			'icon': 'images/icons/net.png',
			'icon_effect': 'tint(alpha=128,red=200,green=200,blue=200);',
			'in_gauge': 'images/gauge/round-rect-white/',
			'in_gauge_effect': 'tint(alpha=255,red=0,green=255,blue=0);',
			'out_gauge': 'images/gauge/round-rect-white/',
			'out_gauge_effect': 'tint(alpha=255,red=255,green=0,blue=0);',
			'separator_color': '00000088'}
	def __init__(self, filename):
		dict.__init__(self)
		self._filename=filename
		self._id='theme'
		self._load_and_save()

class Config(dict):
	def __init__(self,id,file_name):
		self.config=GeneralConfig(id,file_name)
		self.theme_config=None

	def __getitem__(self, key):
		if self.config.has_key(key):
			return self.config[key]
		elif self.theme_config.has_key(key):
			return self.theme_config[key]
			
	def __setitem__(self, key, value):
		if self.config.has_key(key):
			self.config[key]=value
		elif self.theme_config.has_key(key):
			self.theme_config[key]=value
			
	def get_theme(self,file_name):
		if self.theme_config != None:
			del self.theme_config
		self.theme_config=ThemeConfig(file_name)
		
	def color(self,string):
		colors = [eval('0x%s' % string[i*2:i*2+2]) for i in range(len(string)/2)]
		if (len(colors) != 4): colors += [255]
		return colors


class Gauge:
	def __init__(self,canvas, basedir, image_dir, var_name, filter):
		self.canvas=canvas
		self.basedir=basedir
		self.image_dir=image_dir
		self.var_name = var_name
		self.x=self.y=self.value=0
		
		adesklets.set( self.var_name, 0)

		#<# replace this with load_image_without_cache() when it is added to adesklets
		self.image=adesklets.load_image('%s/%s' % (self.basedir, self.image_dir + 'center.png'))
		adesklets.context_set_image(self.image)
		adesklets.clone_image()
		adesklets.free_image(self.image)
		adesklets.context_set_image(self.image)
		if filter:	
			adesklets.apply_filter(filter)
		#># 
		self.bgimage=adesklets.load_image('%s/%s' % (self.basedir, self.image_dir + 'bg.png'))
		adesklets.context_set_image(self.bgimage)
		adesklets.clone_image()
		adesklets.free_image(self.bgimage)
		
		# get image size
		adesklets.context_set_image(self.bgimage)
		self.w=adesklets.image_get_width()
		self.h=adesklets.image_get_height()
	
	def draw(self,background=0):
		adesklets.blend_image_onto_image(self.image,0,
	                                         0,0,'$' + self.var_name,self.h,
					         self.x+self.canvas.x,self.y+self.canvas.y,'$' + self.var_name,self.h)
	def draw_background(self,background):
		#draw gauge background image 
		adesklets.blend_image_onto_image(self.bgimage,1,0,0,self.w,self.h,
					 self.x+self.canvas.x,self.y+self.canvas.y,self.w,self.h)

	
	def update_background(self,background):
		#called when the background changes
		#update gauge foreground image 
		adesklets.context_set_blend(False)
		adesklets.context_set_image(self.image)
		temp_image=adesklets.clone_image()
		adesklets.blend_image_onto_image(background,0,self.x+self.canvas.x, self.y+self.canvas.y,self.w,self.h,0,0,self.w,self.h)
		adesklets.context_set_blend(True)
		adesklets.blend_image_onto_image(temp_image,1,0,0,self.w,self.h,0,0,self.w,self.h)
		adesklets.free_image(temp_image)


class Icon:
	def __init__(self,canvas,basedir,image_file,filter):
		self.canvas=canvas
		self.basedir=basedir
		self.image_file=image_file
		self.x=self.y=self.w=self.h=0

		# get image size
		self.image=adesklets.load_image('%s/%s' % (self.basedir, self.image_file))
		adesklets.context_set_image(self.image)
		if filter:
			adesklets.apply_filter(filter)
		self.w=adesklets.image_get_width()
		self.h=adesklets.image_get_height()
		
		self.aspect_scale(32,32)

	def aspect_scale(self,width,height):
	#needs to be called before any other images are created
		if self.w <= width & self.h <= height: return
	
		if self.w > self.h:
			new_h = self.h * (float(width) / self.w)
			new_w = width
		else:
			new_w = self.w * (float(height) / self.h)
			new_h = height
		
		new_w=int(new_w)
		new_h=int(new_h)
		adesklets.context_set_image(adesklets.create_image(new_w,new_h))
		adesklets.blend_image_onto_image(self.image,1,0,0,self.w,self.h,
		                                              0,0,new_w,new_h)
		adesklets.free_image(self.image)
		self.w=new_w
		self.h=new_h
		
	
	def draw_background(self, background):
		adesklets.context_set_blend(True)
		adesklets.blend_image_onto_image(self.image,1,
	                                         0,0,self.w,self.h,
					         self.x+self.canvas.x,self.y+self.canvas.y,self.w,self.h)

class Text:
	def __init__(self,canvas, font, font_color, text, var_name):
		self.canvas=canvas
		self.var_name=var_name
		adesklets.set( self.var_name, 'Default')
		
		self.font=adesklets.load_font(font)
		self.font_color=font_color
		self.text=text
		
		adesklets.context_set_font(self.font)
		size=adesklets.get_text_size(text)
		self.w=size[0]
		self.h=size[1]
		self.x=0
		self.y=0

	def draw(self,background=0):
		adesklets.context_set_color(*(self.font_color))
		#adesklets.context_set_font(self.font)
		adesklets.text_draw(self.x+self.canvas.x, self.y+self.canvas.y, '$' + self.var_name)

class BackgroundText_align:
	def __init__(self,canvas, font, font_color, text):
		self.canvas=canvas
		
		self.font=adesklets.load_font(font)
		self.font_color=font_color
		self.text=text
		
		adesklets.context_set_font(self.font)
		size=adesklets.get_text_size(text)
		self.w=size[0]
		self.h=size[1]
		self.x=0
		self.y=0
		self.align_right()

	def align_right(self):
		self.xa=self.x-self.w
		self.ya=self.y

	def update_text(self,text):
		self.text = text
		adesklets.context_set_font(self.font)
		size=adesklets.get_text_size(text)
		self.w=size[0]
		self.h=size[1]
		self.align_right()
	
	def update_background(self,background=0):
		adesklets.context_set_image(background)
		adesklets.context_set_color(*(self.font_color))
		adesklets.context_set_font(self.font)
		adesklets.text_draw(self.xa+self.canvas.x, self.ya+self.canvas.y, self.text)


class Canvas:
	def __init__(self, x=0, y=0, w=0, h=0):
		self.x = x
		self.y = y
		self.w = w
		self.h = h

class Events(adesklets.Events_handler):
   
    def __init__(self, basedir):
    	if len(basedir)==0:
    		self.basedir='.'
    	else:
    		self.basedir=basedir
       
	self.top_canvas=Canvas(0,0,200,0)
	self.w = None
	self.h = None
	self.buffer= None
	self.border=None
	self.border_mask=None
	self.get_config()
	self.tx_string=""
	self.rx_string=""
	self.gauge_rx=0
	self.gauge_tx=0
	self.device_list=[]
	
	self.net_in_gauge=Gauge(self.top_canvas,self.basedir,self.config['in_gauge'],'net_in_gauge',self.config['in_gauge_effect'])
	self.net_out_gauge=Gauge(self.top_canvas,self.basedir,self.config['out_gauge'],'net_out_gauge',self.config['out_gauge_effect'])
	self.net_in_text=Text(self.top_canvas,self.config['font'],self.config.color(self.config['font_color']),'IN 1000KB','net_in_text')
	self.net_out_text=Text(self.top_canvas,self.config['font'],self.config.color(self.config['font_color']),'OUT 1000KB','net_out_text')
	self.icon=Icon(self.top_canvas,self.basedir,self.config['icon'],self.config['icon_effect'])
	self.device_name_text=BackgroundText_align(self.top_canvas,self.config['font'],
	                                           self.config.color(self.config['font_color']),self.config['device_name'])
	
	margin=3
	top_margin=3
	bottom_margin=3
	self.icon.x=margin
	
	#### place text
	self.net_in_text.x=self.icon.x+self.icon.w+margin
	self.net_in_text.y=top_margin

      	#### place gauge
	self.net_in_gauge.x=self.net_in_text.x
	self.net_in_gauge.y=self.net_in_text.y+self.net_in_text.h

	#### place background text
	self.device_name_text.x=self.net_in_gauge.x+self.net_in_gauge.w
	self.device_name_text.y=top_margin
	self.device_name_text.align_right()

	### place gauge
	self.net_out_gauge.x=self.net_in_gauge.x
	self.net_out_gauge.y=self.net_in_gauge.y+self.net_in_gauge.h+margin

	#### place text
	self.net_out_text.x=self.net_out_gauge.x
	self.net_out_text.y=self.net_out_gauge.y+self.net_out_gauge.h

	#### calculate height of canvas
	self.top_canvas.h=self.net_out_text.y+self.net_out_text.h + bottom_margin
	
	#### center icon vertically
	self.icon.y =  (self.top_canvas.h/2) - (self.icon.h/2)  

	#### setup our border and border mask
	self.border=self.create_border_image(self.top_canvas,1,0,self.config['border'],self.config['border_effect'])
	adesklets.context_set_image(self.border)
	self.icon.draw_background(self.border)
	adesklets.context_set_image(self.border)
	self.net_in_gauge.draw_background(self.border)
	adesklets.context_set_image(self.border)
	self.net_out_gauge.draw_background(self.border)

	self.border_mask=self.create_border_image(self.top_canvas,0,1,self.config['border'] + 'mask/')
	
	#### fit our window to the border image
	adesklets.context_set_image(self.border)
	self.w = adesklets.image_get_width()
	self.h = adesklets.image_get_height()

	adesklets.Events_handler.__init__(self)

	
    def __del__(self):
        adesklets.Events_handler.__del__(self)
    
    def get_config(self):
    	self.config=Config(adesklets.get_id(),join(self.basedir,'config.txt'))
	self.config.get_theme(join(self.basedir,self.config['theme']))

    def reload_border(self):
	#  this is basically a copy of some code from the init and ready functions
    	#reset the background image to 1
	adesklets.window_set_background_image(1)

    	#free border, border_mask, buffer, and background image
	for i in range(4):
		adesklets.free_image(self.border)
	
	#remove any offset from our canvas
	self.top_canvas.x=0
	self.top_canvas.y=0

	#### create our border image
	self.border=self.create_border_image(self.top_canvas,1,0,self.config['border'],self.config['border_effect'])
	adesklets.context_set_image(self.border)
	
	self.icon.draw_background(self.border)
	adesklets.context_set_image(self.border)
	self.net_in_gauge.draw_background(self.border)
	adesklets.context_set_image(self.border)
	self.net_out_gauge.draw_background(self.border)

	# create border mask image
	self.border_mask=self.create_border_image(self.top_canvas,0,1,self.config['border'] + 'mask/')

	#### fit our window to the border image
	adesklets.context_set_image(self.border)
	self.w = adesklets.image_get_width()
	self.h = adesklets.image_get_height()

	# We use a buffer image to avoid flickering */
	self.buffer=adesklets.create_image(self.w,self.h)

	# Set the window property
        adesklets.window_resize(self.w,self.h)
	sleep(1)
	
	# create alternate background
	self.background=adesklets.create_image(self.w,self.h)
	adesklets.window_set_background_image(self.background)
	
	# should we destroy the old recording? 
	self.record_display()
   
    def ready(self):
	# We use a buffer image to avoid flickering */
	self.buffer=adesklets.create_image(self.w,self.h)
	
	# Set the window property
        adesklets.window_resize(self.w,self.h)
	adesklets.window_set_transparency(False)

	# create alternate background
	self.background=adesklets.create_image(self.w,self.h)
	adesklets.window_set_background_image(self.background)
	
	self.record_display()
	self.create_menu()	
	adesklets.window_show()

    def create_menu(self):
    	adesklets.menu_reset_all()
    	adesklets.menu_add_separator()
	editor=getenv('EDITOR')
	adesklets.menu_add_item('README')	
	adesklets.menu_add_submenu('Configure')
	if editor:
		adesklets.menu_add_item('Edit_Config')
	adesklets.menu_add_submenu('BorderToggle')
	adesklets.menu_add_item('Left')
	adesklets.menu_add_item('Right')
	adesklets.menu_add_item('Top')
	adesklets.menu_add_item('Bottom')
	adesklets.menu_end_submenu()
	adesklets.menu_add_submenu('SetDevice')
	self.get_device_list()
	for device in self.device_list:
		adesklets.menu_add_item(device)	
	adesklets.menu_end_submenu()
	adesklets.menu_end_submenu()


    def menu_fire(self, delayed, menu_id, item, something=0):
	if item=='Edit_Config':
		editor=getenv('EDITOR')
		system('xterm -e %s %s/config.txt &' % (editor, self.basedir ))
	elif item=='README':
		system('xterm -e less %s/README &' % (self.basedir))
	elif item=='Left':
		if self.config['border_left'] == True:
			self.config['border_left'] = False
		else:
			self.config['border_left'] = True
		self.reload_border()
   	elif item=='Right':
		if self.config['border_right'] == True:
			self.config['border_right'] = False
		else:
			self.config['border_right'] = True
		self.reload_border()
	elif item=='Top':
		if self.config['border_top'] == True:
			self.config['border_top'] = False
		else:
			self.config['border_top'] = True
		self.reload_border()
	elif item=='Bottom':
		if self.config['border_bottom'] == True:
			self.config['border_bottom'] = False
		else:
			self.config['border_bottom'] = True
		self.reload_border()
	elif item in self.device_list:
		self.config['device_name']=item
	
	
    def create_border_image(self, canvas, offset_canvas, mask, directory, filter=None):
	"""
	create border image around outside of canvas
	"""
	#load images
    	tl=adesklets.load_image('%s/%s' % (self.basedir, directory + 'top-left.png'))
    	tr=adesklets.load_image('%s/%s' % (self.basedir, directory + 'top-right.png'))
    	bl=adesklets.load_image('%s/%s' % (self.basedir, directory + 'bottom-left.png'))
    	br=adesklets.load_image('%s/%s' % (self.basedir, directory + 'bottom-right.png'))
	l=adesklets.load_image('%s/%s' % (self.basedir, directory + 'left.png'))
	r=adesklets.load_image('%s/%s' % (self.basedir, directory + 'right.png'))
	t=adesklets.load_image('%s/%s' % (self.basedir, directory + 'top.png'))
	b=adesklets.load_image('%s/%s' % (self.basedir, directory + 'bottom.png'))
	
	#get border sizes
	adesklets.context_set_image(l)
	l_h=adesklets.image_get_height()
	l_w=adesklets.image_get_width()
	adesklets.context_set_image(r)
	r_h=adesklets.image_get_height()
	r_w=adesklets.image_get_width()
	adesklets.context_set_image(t)
	t_w=adesklets.image_get_width()
	t_h=adesklets.image_get_height()
	adesklets.context_set_image(b)
	b_w=adesklets.image_get_width()
	b_h=adesklets.image_get_height()	
	
	#apply our filter
	if filter!=None:
		for id in range(tl,b+1):
			adesklets.context_set_image(id)
			adesklets.apply_filter(filter)
	
	#calculate destination image size and offset canvas if requested
	w=canvas.w
	h=canvas.h
	if self.config['border_top']==True:
		h+=t_h
		if offset_canvas:
			canvas.y += t_h
	if self.config['border_bottom']==True:
		h+=b_h
	if self.config['border_left']==True:
		w+=l_w
		if offset_canvas:
			canvas.x += l_w
	if self.config['border_right']==True:
		w+=r_w
	
	#create image
	image=adesklets.create_image(w,h)
	adesklets.context_set_blend(False)
	adesklets.context_set_image(image)
	if mask:
		adesklets.context_set_color(0,0,0,255)
	else:
		adesklets.context_set_color(0,0,0,0)
	adesklets.image_fill_rectangle(0,0,w,h)
	
	#draw sides
	if self.config['border_top']==True:
		adesklets.blend_image_onto_image(t,1,0,0,t_w,t_h,
		                                     0,0,w,t_h)
	if self.config['border_bottom']==True:
		adesklets.blend_image_onto_image(b,1,0,0,b_w,b_h,
		                                     0,h-b_h,w,b_h)
	if self.config['border_left']==True:
		adesklets.blend_image_onto_image(l,1,0,0,l_w,l_h,
		                                     0,0,l_w,h)
	if self.config['border_right']==True:
		adesklets.blend_image_onto_image(r,1,0,0,r_w,r_h,
		                                     w-r_w,0,r_w,h)
	#draw corners
	if self.config['border_top']==True & self.config['border_left']==True:
		adesklets.context_set_image(tl)
		corner_w=adesklets.image_get_width()
		corner_h=adesklets.image_get_height()
		adesklets.context_set_image(image)
		adesklets.blend_image_onto_image(tl,1,0,0,corner_w,corner_h,0,0,corner_w,corner_h)
	if self.config['border_top']==True & self.config['border_right']==True:
		adesklets.context_set_image(tr)
		corner_w=adesklets.image_get_width()
		corner_h=adesklets.image_get_height()
		adesklets.context_set_image(image)
		adesklets.blend_image_onto_image(tr,1,0,0,corner_w,corner_h,w-corner_w,0,corner_w,corner_h)
	if self.config['border_bottom']==True & self.config['border_left']==True:
		adesklets.context_set_image(bl)
		corner_w=adesklets.image_get_width()
		corner_h=adesklets.image_get_height()
		adesklets.context_set_image(image)
		adesklets.blend_image_onto_image(bl,1,0,0,corner_w,corner_h,0,h-corner_h,corner_w,corner_h)
	if self.config['border_bottom']==True & self.config['border_right']==True:
		adesklets.context_set_image(br)
		corner_w=adesklets.image_get_width()
		corner_h=adesklets.image_get_height()
		adesklets.context_set_image(image)
		adesklets.blend_image_onto_image(br,1,0,0,corner_w,corner_h,w-corner_w,h-corner_h,corner_w,corner_h)
	#draw our separator
	if (self.config['border_bottom']!=True) & (mask!=True):
		adesklets.context_set_image(image)
		adesklets.context_set_blend(True)
		adesklets.context_set_color(*self.config.color(self.config['separator_color']))
		if self.config['border_left']!=True:
			l_w=0
		if self.config['border_right']!=True:
			r_w=0
		adesklets.image_draw_line(l_w+15,h-1,(w-r_w)-15,h-1,1)
	
	#free our temp images
	for i in range(8):
        	adesklets.free_image(tl)

	return image-8


    def background_grab(self, delayed):
	#update our alternate background image
	adesklets.context_set_image(self.background)
	adesklets.blend_image_onto_image(1,1,0,0,self.w,self.h,0,0,self.w,self.h)
    	
	#copy background image to buffer and apply effects
	adesklets.context_set_blend(False)
	adesklets.context_set_image(self.buffer)
	adesklets.blend_image_onto_image(self.background,1,0,0,self.w,self.h,0,0,self.w,self.h)
	adesklets.apply_filter(self.config['background_effect'])
	adesklets.image_blur(self.config['background_blur'])

	#blend modified background onto our bordermask
	adesklets.context_set_blend(True)
	adesklets.context_set_image(self.border_mask)
	adesklets.blend_image_onto_image(self.buffer,0,0,0,self.w,self.h,0,0,self.w,self.h)
	
	#combine everything 
	adesklets.context_set_blend(True)
	adesklets.context_set_image(self.background)
	adesklets.blend_image_onto_image(self.border_mask,0,0,0,self.w,self.h,0,0,self.w,self.h)
	adesklets.blend_image_onto_image(self.border,1,0,0,self.w,self.h,0,0,self.w,self.h)
	self.net_in_gauge.update_background(self.background)
	self.net_out_gauge.update_background(self.background)
	self.device_name_text.update_background(self.background)

	#copy our new background to the foreground image
	adesklets.context_set_image(0)
	adesklets.blend_image_onto_image(self.background,1,0,0,self.w,self.h,0,0,self.w,self.h)
	
	adesklets.play(*self.display)
	
    def record_display(self):
	adesklets.start_recording()
	#adesklets.context_set_blend(False)
	adesklets.context_set_image(self.buffer)
	adesklets.blend_image_onto_image(self.background,0,self.top_canvas.x+32,self.top_canvas.y,self.top_canvas.w-32,self.top_canvas.h,
                                                           self.top_canvas.x+32,self.top_canvas.y,self.top_canvas.w-32,self.top_canvas.h)
	self.net_in_gauge.draw(self.buffer)
	self.net_out_gauge.draw(self.buffer)
	self.net_in_text.draw(self.buffer)
	self.net_out_text.draw(self.buffer)
	
	adesklets.context_set_image(0)
	adesklets.blend_image_onto_image(self.buffer,0,self.top_canvas.x+32,self.top_canvas.y,self.top_canvas.w-32,self.top_canvas.h,
                                                       self.top_canvas.x+32,self.top_canvas.y,self.top_canvas.w-32,self.top_canvas.h)

	self.display=adesklets.stop_recording()
	adesklets.play_set_abort_on_events(False)


    def quit(self):
        print 'Quitting...'
	
    def get_device_list(self):
	self.device_list=[]
	stats=statgrab.sg_get_network_io_stats_diff()
	for device in stats:
		self.device_list.append(device['interface_name'])


    def get_net_stats(self):
	self.last_tx_string = self.tx_string
	self.last_rx_string = self.rx_string
	self.last_gauge_rx = self.gauge_rx
	self.last_gauge_tx = self.gauge_tx
	
	temp_list=[]
	device=None

	stats = statgrab.sg_get_network_io_stats_diff()
		
	# get the device list and select our device 
	found_device=False
	for device_temp in stats:
		temp_list.append(device_temp['interface_name'])
		if device_temp['interface_name']==self.config['device_name']:
			device=device_temp
			found_device=True	
	
	# if our device list changes update menu
	if temp_list != self.device_list:
		self.create_menu()
	 
	if not found_device:
		self.rx_string = "rx: ----"
		self.tx_string = "tx: ----"
		self.gauge_rx = 0
		self.gauge_tx = 0
		if self.device_name_text.text!=self.config['device_name']:
			self.device_name_text.update_text(self.config['device_name'])
			self.background_grab(0)
		return
	
	# if the device changed, update window text 
	if self.device_name_text.text!=device['interface_name']:
		self.device_name_text.update_text(device['interface_name'])
		self.background_grab(0)
	
	systime=max(1.0,float(device['systime']))

	rx=(device['rx']/1024)/systime
	tx=(device['tx']/1024)/systime
	
	if rx<999 :
		self.rx_string = 'rx: %s%s' % (int(rx) , "KB/s")
	else:
		self.rx_string = 'rx: %.2f%s' % (float(rx/1024.0) , "MB/s")

	if tx<999 :
		self.tx_string = 'tx: %s%s' % (int(tx) , "KB/s")
	else:
		self.tx_string = 'tx: %.2f%s' % (float(tx/1024.0) , "MB/s")
	
	if rx > self.config['max_KB_in']:
		rx = self.config['max_KB_in']
	rx *= float(self.net_in_gauge.w / float(self.config['max_KB_in']))
	self.gauge_rx = int(rx)
	
	if tx > self.config['max_KB_out']:
		tx = self.config['max_KB_out']
	tx *= float(self.net_out_gauge.w / float(self.config['max_KB_out']))
	self.gauge_tx = int(tx)
	
	
    def alarm(self):
	self.block()
	self.get_net_stats()
	
	# draw to the window only if something changes
	if (self.last_tx_string!=self.tx_string) | (self.last_rx_string!=self.rx_string) |\
	   (self.last_gauge_rx!=self.gauge_rx) | (self.last_gauge_tx!=self.gauge_tx):
		adesklets.set( self.net_in_gauge.var_name, self.gauge_rx)
		adesklets.set( self.net_in_text.var_name, self.rx_string)
		adesklets.set( self.net_out_gauge.var_name, self.gauge_tx)
		adesklets.set( self.net_out_text.var_name, self.tx_string)
		adesklets.play(*self.display)

	self.unblock()
	
	return  1
    
Events(dirname(__file__)).pause()
